//=============================================================================
// RPG Maker MZ - TPitemPlugin
//=============================================================================

/*:
 * @target MZ
 * @author Evilcat
 * @plugindesc Custom TP Item Functionality
 * 
 * @help This plugin allows items or skill to grant or reduce TP to target actor.
 * Disable tp random up function when start battle and disable tp reset when battle end. 
 * Also showing the TP gauge on the menu outside of battle.
 * 
 * To use this plugin, include the following in the item or skill note field:
 * <tp:n> - where n is the amount of TP to be gained (positive or negative).
 * 
 * Example:
 * <tp:10>  - Grants 10 TP to the selected character.
 * <tp:-5>  - Reduces 5 TP from the selected character.
 * 
 * Note: In addition to putting <tp:n> in the note, you need to add a dummy state according to dummyStatesId.
 * (Example: Dummy TP) This will allow TP item and Skill to be use on actor (the dummy states will be remove immediately).
 * 
 * @param TP100StatesId
 * @desc Add States ID when TP become 100.
 * @type state
 * @default 9
 * 
 * @param dummyStatesId
 * @desc The States ID to use ass dummy.
 * @type state
 * @default 10
 * 
 * @command setTP
 * @text Set TP of actor.
 * @desc Set TP of actor.
 * 
 * @arg value
 * @type number
 * @min 0
 * @max 100
 * @text TP value
 * @desc The TP value.
 * 
 * @arg actor
 * @type actor
 * @text target actor
 * @desc The actor to change TP.
 * 
 * 
 * @command gainTP
 * @text Gain TP of actor.
 * @desc Gain TP of actor.
 * 
 * @arg value
 * @type number
 * @min -100
 * @max 100
 * @text TP value
 * @desc The TP value.
 * 
 * @arg actor
 * @type actor
 * @text target actor
 * @desc The actor to change TP.
 * 
 */

(() => {

    let param = PluginManager.parameters("TPitemPlugin");
    let TP100StatesId = Number(param["TP100StatesId"]) || 9;
    let dummyStatesId = Number(param["dummyStatesId"]) || 10;

    // Override applyItemUserEffect to apply TP to the selected target.
    Game_Action.prototype.applyItemUserEffect = function(target) {
        const value = Math.floor(this.item().tpGain * this.subject().tcr);
	this.subject().gainSilentTp(value);
        if (this.item().meta.tp != undefined) {
            target.gainSilentTp(parseInt(this.item().meta.tp));
        }
    };

    Game_Action.prototype.itemEffectAddNormalState = function(target, effect) {
    	let chance = effect.value1;
    	if (!this.isCertainHit()) {
            chance *= target.stateRate(effect.dataId);
            chance *= this.lukEffectRate(target);
    	}
    	if (Math.random() < chance) {
            target.addState(effect.dataId);
            this.makeSuccess(target);
	    if(effect.dataId == dummyStatesId)target.removeState(dummyStatesId);
    	}
    };

    Game_BattlerBase.prototype.setTp = function(tp) {
	if(!this.isStateAffected(TP100StatesId)){
	    this._tp = tp;
	    if(this._tp>=100)this.addState(TP100StatesId);
	    this.refresh();
	}
    };

    Game_Battler.prototype.chargeTpByDamage = function(damageRate) {
    	const value = Math.floor(50 * damageRate * this.tcr);
    };

    Game_Battler.prototype.onBattleStart = function(advantageous) {
        this.setActionState("undecided");
        this.clearMotion();
        this.initTpbChargeTime(advantageous);
        this.initTpbTurn();
    };

    Game_Battler.prototype.onBattleEnd = function() {
        this.clearResult();
        this.removeBattleStates();
        this.removeAllBuffs();
        this.clearActions();
        this.appear();
    };

    Sprite_Gauge.prototype.isValid = function() {
        if (this._battler) {
            return true;
        }
        return false;
    };

    PluginManager.registerCommand("TPitemPlugin", "setTP", args => {
	$gameActors.actor(parseInt(args.actor)).setTp(parseInt(args.value));
    });

    PluginManager.registerCommand("TPitemPlugin", "gainTP", args => {
	$gameActors.actor(parseInt(args.actor)).gainSilentTp(parseInt(args.value));
    });

})();